//===========================================================
// functions to deal with web page input
// search for a string of the form key=value in
// a string that looks like q?xyz=abc&uvw=defgh HTTP/1.1\r\n
//
uint8_t find_key_val(char *str, char *key)
{
  char *kp;
  uint8_t found = 0;
  uint8_t l = 0;
  
  kp = key;
  while (*str &&  *str != ' ' && found == 0) {
    if (*str == *kp) {
      kp++;
      if (*kp == '\0') {
        str++;
        kp = key;
        if (*str == '=') {
          found = 1;
        }
      }
    } else {
      kp = key;
    }
    str++;
  }
  if (found == 1) {
    // copy the value to a buffer and terminate it with '\0'
    while (*str &&  *str != ' ' && *str != '&' && l < STR_BUFFER_SIZE) {
      strbuf[l] = *str;
      l++;
      str++;
    }
    strbuf[l] = '\0';
  }
  // return the length of the value
  return(l);
}

// add a decimal point if needed: 120 becomes 12 and 105 becomes 10.5
// 10 becomes 1 and 11 becomes 1.1
// string must be at least 2 char long.
void adddotifneeded(char *s)
{
  int8_t l = strlen(s);
  
  if (l < 2) {
    return;
  }
  // look at last digit:
  if(s[l - 1] == '0') {
    s[l - 1] = '\0';
  }
  else{
    // add dot
    s[l] = s[l - 1];
    s[l-1] = '.';
    s[l+1] = '\0';
  }
}

// delete any decimal point. If no decimal point is available then add a zero at the end.
void deldot(char *s)
{
  char c;
  char *dst;
  uint8_t founddot = 0;
  
  dst = s;
  while ((c = *s)) {
    if (c == '.' || c == ',') { // dot or comma
      s++;
      founddot = 1;
      continue;
    }
    *dst = c;
    dst++;
    s++;
  }
  if (founddot == 0) {
    *dst = '0';
    dst++;
  }
  *dst = '\0';
}

// take string in dot-ed notation and make a binary IP addr array
boolean str2ip(uint8_t *resultip, char *str)
{
  uint8_t l = 0;
  uint8_t ip[4];
  
  char *strptr;
  while (*str && l <4) {
    // set to first digit (clean leading garbabge:
    while (!isdigit(*str) && *str) {
      if ( *str != ',' ) l = 99;
      str++;
    }
    strptr = str;
    while (isdigit(*str) && *str) {
      str++;
    }
    if (*str) {
      // terminate string unless we are at the end
      // generally we will replace the dot with a \0
      *str = '\0'; 
      str++;
    }
    ip[l] = atoi(strptr);
    l++;
  }
  if (l == 4) {
    // result ok, copy
    l = 0;
    while (l < 4) {
      resultip[l] = ip[l];
      l++;
    }
    return true;
  } else {
    return false;
  }
}

// take binary IP addr array and display in dot-ed notation
void ip2str(char *resultstr, uint8_t *ip)
{
  uint8_t l = 0;
  uint8_t m = 0;
  
  while (l < 4) {
    itoa((int)ip[l], &resultstr[m], 10);
    // search end of str:
    while (resultstr[m]) {
      m++;
    }
    resultstr[m] = '.';
    m++;
    l++;
  }
  m--;
  resultstr[m] = '\0';
}
// Convert key number
int8_t get_key() {

 int8_t NUM_KEYS = 5;

 // Instantiate a Bounce object with a 5 millisecond debounce time
 Bounce sw_ch_1 = Bounce( SW_CH1, DEBOUNCE );
 Bounce sw_ch_2 = Bounce( SW_CH2, DEBOUNCE );
 Bounce sw_ch_3 = Bounce( SW_CH3, DEBOUNCE );
 Bounce sw_ch_4 = Bounce( SW_CH4, DEBOUNCE );
 Bounce sw_ch_5 = Bounce( SW_CH5, DEBOUNCE );

  // Update the debouncer
  sw_ch_1.update();
  sw_ch_2.update();
  sw_ch_3.update();
  sw_ch_4.update();
  sw_ch_5.update();

 // Get the update value
 if ( sw_ch_1.read() == LOW ){ NUM_KEYS = 0;
 } else if (sw_ch_2.read() == LOW ) { NUM_KEYS = 1;
 } else if (sw_ch_3.read() == LOW ) { NUM_KEYS = 2;
 } else if (sw_ch_4.read() == LOW ) { NUM_KEYS = 3;
 } else if (sw_ch_5.read() == LOW ) { NUM_KEYS = 4;
 } else {
   NUM_KEYS = -1;
 }

 return NUM_KEYS;
}
// 7SegmentLED DP Flashing
void dp_flash(uint8_t fl_mod, uint16_t fl_wait) {
  
  for ( int8_t l = 0; l < 2; l++ ) {
    for ( int8_t m = 0; m < 6; m++ ) {
      if ( m % fl_mod == 0 ) {
       lc.setChar( 0, m, ' ', l );
      }
    }
    delay( fl_wait );
  }
  lc.clearDisplay( 0 );
  delay( fl_wait );
  
  return;
}
// 7SegmentLED IP-Display
void ipscrollDigits(uint8_t *ip){
  
  static int8_t strseg[11];
  static boolean dotflag[] = { 0,0,1,0,0,1,0,0,1,0,0,0 };
  
  for (uint8_t k=0; k<4; k++){
    strseg[ k * 3 ] = ip[k] / 100;
    strseg[ k * 3 + 1 ] = ( ip[k] % 100 ) / 10;
    strseg[ k * 3 + 2 ] = ( ip[k] % 100 ) % 10;
  }
  
  // DebugPrint
  /*for (uint8_t k=0; k<11+1; k++){
    Serial.print(strseg[k]);
    if ( k==2 || k==5 || k==8 ){
      Serial.print(".");
    }
  }
  Serial.println("");*/
  
  lc.clearDisplay( 0 );
  for ( int8_t k = -5; k < 12 + 6; k++ ){
    for ( int8_t m = 0; m < 6; m++ ) {
      if ( k - m >= 0 ){
        if ( k - m < 11 + 1){
          lc.setChar( 0, m, strseg[ k - m ], dotflag[ k - m ] );
        }else{
          lc.setChar( 0, m, ' ', true );
        }
      }
    }
    delay( 500 );
  }
  return;
}
// OneWire DS18x20, Temperature Reading
float ReadTemp(){
  
  static byte dataC[12];
  static float celsius, fahrenheit;

  ds.reset();
  ds.select(addr);
  ds.write(0x44,1);  // start conversion, with parasite power on at the end
  
  ds.reset();
  ds.select(addr);    
  ds.write(0xBE);  // Read Scratchpad 
  
  for ( uint8_t i = 0; i < 9; i++) {           // we need 9 bytes
    dataC[i] = ds.read();
  }
  
  // convert the data to actual temperature
  unsigned int raw = (dataC[1] << 8) | dataC[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (dataC[7] == 0x10) {
      // count remain gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - dataC[6];
    }
  } else {
    byte cfg = (dataC[4] & 0x60);
    if (cfg == 0x00) raw = raw << 3;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw << 2; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw << 1; // 11 bit res, 375 ms
    // default is 12 bit resolution, 750 ms conversion time
  }
  celsius = (float)raw / 16.0;
  //fahrenheit = celsius * 1.8 + 32.0;
  
  return celsius;
  
}
// H values ​​of HSV, the analog output of each LED (0-255) process to convert
uint8_t hsv_rgb(uint16_t Hsv, uint8_t *cMap)
{
  static uint8_t Red, Green, Blue;
  
  if (Hsv <= 120) {
       /* H_Value(0-120) Red^Yellow-Green */
       Red = map(Hsv,0,120,255,0);     // Red_LED R←→G
       Green = map(Hsv,0,120,0,255);     // Green_LED G←→R
       Blue = 0;
  } else if (Hsv <= 240) {
       /* H_Value(120-240) Green-Skyblue-Blue */
       Green = map(Hsv,120,240,255,0);   // Green_LED G←→B
       Blue = map(Hsv,120,240,0,255);   // Blue_LED B←→G
       Red = 0;
  } else {
       /* H_Value(240-360) Blue-Purple-Red */
       Blue = map(Hsv,240,360,255,0);   // Blue_LED B←→R
       Red = map(Hsv,240,360,0,255);   // Red_LED R←→B
       Green = 0;
  }
  cMap[0] = Red;
  cMap[1] = Green;
  cMap[2] = Blue;
  
  return *cMap;
}
