/*
 * AVR NTP Clock(Arduino + ENC28J60 + RTC + 7segment-LED(MAX7219)*6
 * Which is supported Ardunio ATMega644P & 1284P.
 * RTC has power backup using the DS1307 or RTC8564.
 */

#include <LedControl.h> // 7Seg-LED/MatrixLED(MAX7219)
#include <avr/EEPROM.h> // EEPROM Read/Write
#include <RTClib.h>     // DS1307 or RTC-8564 RealTimeClock
#include <Wire.h>       // TWI/I2C(RTC)
#include <OneWire.h>    // DS18B20 Temperature
#include <Bounce.h>     // Switch input(BounceCheck)

// Use different pins to 2,10,11,12,13 as they are used by ethershield
// Pins D2:INT/D10:CS/D11:MOSI/D12:MISO/D13:SCK
#include <EtherShield.h>

#include <EthPacketDump.h> // ENC28J60 EtherShield EthPacketDump
EthPacketDump dump=EthPacketDump();

// LedControl(MAX72XX)
// LedControl(X1,X2,X3,X4)
// X1:DataIn X2:CLK X3:LOAD X4:Unit
//LedControl lc = LedControl( 21, 20, 19, 1);
LedControl lc=LedControl( 19, 21, 20, 1);

// OneWire DS18S20, Temperature Sensor
#define TEMP_SENSOR_OFFSET 1;
OneWire  ds(18); // PC2:D18

// Pin to ground to reset EEPROM values(DegitalPIN)
#define RESET_PIN 24 // PA7:AI7

// This is a LED display is for handling interrupts per 1second.(DegitalPIN)
#define SEC_LED 23 // PC7:D23

// This is a LED status display is for NTP.(DegitalPIN)
#define NTP_LED 22 // PC6:D22

// This is a RGB-LED output pin configuration
#define LED_ENA 0
#define LED_PIN_RED 12
#define LED_PIN_GREEN 13
#define LED_PIN_BLUE 14

// This is a SW_5ch(AnalogPIN)
#define SW_CH1 30 // AI1(12H/24H)
#define SW_CH2 29 // AI2(Set)
#define SW_CH3 28 // AI3(Select/Date)
#define SW_CH4 27 // AI4(Up/Zero+)
#define SW_CH5 26 // AI5(Down/Zero)
#define DEBOUNCE 5 // debounce time(ms)

// This is a Photo Ics(AnalogPIN)
#define PHOTO_SENSOR 0 // AI2

// This is a sensors and pulse for Step-up circuit
#define HV_SEN_PIN 0
#define HV_PWM_PIN 15

// 抵抗分圧回路
// HV--R1(39k)--SENS--R2(1k)--GND
// 測定可能電圧0-200V
#define HV_ADJ 0.9 // 誤差処理
#define HV_SET_VOLT 150

// bit set / clear
#ifndef cbi
#define cbi(PORT, BIT) (_SFR_BYTE(PORT) &= ~_BV(BIT))
#endif
#ifndef sbi
#define sbi(PORT, BIT) (_SFR_BYTE(PORT) |= _BV(BIT))
#endif

// RTC Setting
RTC_DS1307 RTC; //DS1307 64 x 8, Serial, I2C Real-Time Clock
//RTC_RTC8564 RTC; //RTC-8564 JE/NB I2C Real-Time Clock
#define RTC_INIT_TIME 500000 //RTC-8564 JE/NB I2C Real-Time Clock

// --------------- debug print
#define DEBUG

#ifdef DEBUG
  #define DEBUG_PRINT(x) Serial.print(x)
  #define DEBUG_PRINTH(x,h) Serial.print(x,h)
  #define DEBUG_PRINTLN(x) Serial.println(x)
  #define DEBUG_PRINTLNH(x,h) Serial.println(x,h)
#else
  #define DEBUG_PRINT(x)
  #define DEBUG_PRINTH(x,h)
  #define DEBUG_PRINTLN(x)
  #define DEBUG_PRINTLNH(x,h)
#endif

// --------------- modify start
// Some message of introduction
static const char strTitle1[] PROGMEM = "7Segment-LED NTP/RTC Clock Ver1.00b";

// Default Network Congfig
static uint8_t mac_address[6] = { 0x54, 0x55, 0x58, 0x10, 0x01, 0x28 }; // Mac
static uint8_t ip_address[4] = { 192, 168, 12, 1 }; // IP(html)
static uint8_t netmask[4] = { 255, 255, 255, 0 }; // SN(html)

// Default gateway
static uint8_t gw_addrsss[4] = { 0, 0, 0, 0 }; // GW(html)

// DNS server IP.
static uint8_t dns_address[4] = { 0, 0, 0, 0 }; // NS(html)

// Populated as part of DHCP address allocation
static uint8_t dhcpsvrip[4] = { 0, 0, 0, 0 };

// Between sending a DHCP request(min)
#define DHCP_POL_INTERVAL 360

// listen port for tcp/www:
#define MYWWWPORT 80

// Default TimeServer Setting
static uint8_t ntp_polling_address[4] = { 133, 243, 238, 164 }; // ntp.nict.jp , NTA(html)
//static uint8_t ntp_polling_address[4] = { 133, 243, 238, 244 }; // ntp.nict.jp , NTA(html)
//static uint8_t ntp_polling_address[4] = { 210, 173, 160, 27 }; // ntp1.jst.mfeed.ad.jp , NTA(html)
//static uint8_t ntp_polling_address[4] = { 210, 173, 160, 57 }; // ntp2.jst.mfeed.ad.jp , NTA(html)
//static uint8_t ntp_polling_address[4] = { 210, 173, 160, 87 }; // ntp3.jst.mfeed.ad.jp , NTA(html)

// Time Zone Setting(DefaultValue)
static int8_t hours_offset_to_utc = 90;  // TZ(html) 

// Initial Staring(DefaultValue)
boolean clock_disp = true;              // ClockDisplay , DISP(html)
static boolean cefa = true;             // TCF(html)
boolean dimmer = false;                 // DM(html)
boolean night_mode = false;             // NM(html)
static boolean ampm = true;             // 12H/24H , AP(html)
//static boolean network = true;        // Ethernet(html)
static boolean dhcp = true;             // DHCP(html)
static boolean ntp = true;              // NTP(html)
//static boolean temp_disp;             // TD(html)

uint16_t date_disp_int = 1;            // DDI(html)
static uint8_t date_disp_sec = 5;      // DDS(html)
static uint8_t colon_disp  = 1;        // CD(html)
static uint8_t day_brightness = 15;    // DB(html)
static uint8_t night_brightness = 1;   // NB(html)
static uint8_t day_time_start = 6;     // DTS(html)
static uint8_t day_time_end = 18;      // DTE(html)

static uint8_t LED_mode = 0;           // LM(html)
static uint8_t RGB_change_speed = 0;   // LCS(html)
static uint8_t LED_red = 0;            // LR(html)
static uint8_t LED_green = 0;          // LG(html)
static uint8_t LED_blue = 0;           // LB(html)

//static uint8_t LED_light_time_start = 0;     // LTS(html)
//static uint8_t LED_light_time_end = 0;       // LTE(html)

static float  tempMin    = 10;                 // MinTemp
static float  tempMax    = 35;                 // MaxTemp

// NTP Polling Interval(DefaultValue)
//static uint8_t ntpint[] = { 1, 2, 3, 5, 15, 30 };  // IntervalTime
static uint8_t ntpint[] = { 5, 15, 30, 60, 120, 255 };  // IntervalTime
static uint8_t ntp_interval = (int8_t)60;        // NTI(html)

// This is a DimmerSW Down Brightness
#define DIMMER_DOWN 5

// --------------- modify stop

// Ethernet(ENC28J60) InitialSetting
#define BUFFER_SIZE 1350
static uint8_t buf[BUFFER_SIZE + 1];

// Setup the EtherShield class
EtherShield es = EtherShield();

// RTC InitialSetting
static boolean TimeKeep;
static uint16_t prev_min = 9999;

// DS18x20 InitialSetting
static  byte type_s;
static byte addr[8];

// we do not update LCD from interrupt, we do it when we have time (no ip packet)
static uint8_t lcd_needs_update = 1;

// global variable
boolean SET_MODE = false;
static uint8_t SEL_CAT = 0;
static uint8_t SEL_VAL = 0;
static boolean SetBlink = false;
static boolean NTP_LED_WaitFlag = false;

static int8_t dns_state = -1;
static uint8_t ntpclientportL = 0;

// global string buffer
#define STR_BUFFER_SIZE 24
static char strbuf[STR_BUFFER_SIZE + 1];
static char dnsreqesthost[STR_BUFFER_SIZE + 1];

// Static Strings in PROGMEM
static const char strWaitingFor[] PROGMEM = "Waiting for";
static const char strCheckingLink[] PROGMEM = "Checking Link";
static const char strHttpHeader[] PROGMEM = "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nPragma: no-cache\r\n\r\n";
static const char strPageHeader[] PROGMEM = "<pre>";
static const char strPageFooter[] PROGMEM = "</pre>";

static const char strhtmlPageHeader[] PROGMEM = "<html>\n";
static const char strhtmlPageTitle[] PROGMEM = "<head>\n<title>Arduino Ethernet Clock</title>\n</head>\n";
//static const char strhtmlPageTitle[] PROGMEM = "<head>\n<META http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\">\n<title>Arduino Ethernet Clock</title>\n</head>\n";
static const char strhtmlPageHeader2[] PROGMEM = "<body>\n<center><h1>";
static const char strhtmlPageHeader3[] PROGMEM = "</h1></center>\n<hr><br>\n<center>\n<table border=1 width=400 cellspacing=0 cellpadding=5 bordercolor=#333333>";
static const char strhtmlPageFooter[] PROGMEM = "</body>\n</html>";
static const char strhtmlbr[] PROGMEM = "<br>\n";
static const char strhtmlSettingPageLink[] PROGMEM = "</td></tr>\n</table>\n<table border=0 width=400>\n<tr><td>\n<div align=right>\n<input type=hidden name=np><input type=hidden name=fd value=1>\n<input type=submit value=Save></form>\n</div>\n</td></tr>\n</table>\n<hr></center>\n<a href=\"../config\"><i>=> Show all settings</i></a>";

static const char strPageLink1[] PROGMEM = "<a href=\"../config1\">=>Clock</A>\n";
static const char strPageLink2[] PROGMEM = "<a href=\"../config2\">=>Network</A>\n";
static const char strPageLink3[] PROGMEM = "<a href=\"../config3\">=>LED</A>\n";
static const char strPageSubmit[] PROGMEM = "<input type=hidden name=np><input type=hidden name=fd value=1>\n<input type=submit value=Save></form>\n";

// WeekDayConvert
static const char day_abbrev[] PROGMEM = "SunMonTueWedThuFriSat";

// Display AM/PM
static const char ampm_abbrev[] PROGMEM = "AMPM";

// Waiting Indicator
//static const char wait_abbrev[] PROGMEM = "UGWN?M";
static const char wait_abbrev[] PROGMEM = "JvC$m]";
//static const char wait_abbrev[] PROGMEM = "%^7VuL";
//static const char wait_abbrev[] PROGMEM = ""{`1jxI";

// Display linkstatus
static uint8_t linkstatus[6] = { 0, 0, 0, 0, 0, 0 };

// Number of seconds between 1-Jan-1900 and 1-Jan-1970, unix time starts 1970 and ntp time starts 1900.
#define TIMEOFDAY_TO_NTP_OFFSET 2208988800
#define TIMEOFDAY_TO_NTP_OFFSET_2036 2085978496
