// Program Initial Setup
void setup()
{
  // Setup Serial
  Serial.begin(19200);
  
  // StartupMessage
  strcpy_P( strbuf, strTitle1 );
  DEBUG_PRINTLN( strbuf ); // debug print
  
  // Setup RealTimeClock
  Wire.begin();
  TimeKeep = RTC.isrunning();
  
  if ( !TimeKeep ) {
    // RTC Starting
    RTC.begin();
    RTC.adjust(DateTime(2000,01,01,00,00,00)); // Format : Year, Month, Day, Hour, Min, Sec
    DEBUG_PRINTLN("RTC TimeLost"); // debug print
  } else {
    DateTime now = RTC.now(); // RTC Getting Time
    DEBUG_PRINTLN("RTC TimeKeep"); // debug print
    DEBUG_PRINT((int16_t)now.year());
    DEBUG_PRINT('/');
    DEBUG_PRINT((int8_t)now.month());
    DEBUG_PRINT('/');
    DEBUG_PRINT((int8_t)now.day());
    DEBUG_PRINT(' ');
    DEBUG_PRINT((int8_t)now.hour());
    DEBUG_PRINT(':');
    DEBUG_PRINT((int8_t)now.minute());
    DEBUG_PRINT(':');
    DEBUG_PRINTLN((int8_t)now.second());
    //DEBUG_PRINT("UnixTime:");
    //DEBUG_PRINTLN((int32_t)now.unixtime());
  }
  // RTC CLKOUT-Setting
  if( RTC.clkout( 3 ) ) { // RTC CLKOUT(1hz)
    DEBUG_PRINTLN("Clock Start"); // debug print
  } else {
    DEBUG_PRINTLN("Clock fails to start"); // debug print
  }
  
  // Initial Setup
  pinMode( RESET_PIN, INPUT ); // config resetting
  digitalWrite( RESET_PIN, HIGH ); // Enable internal pull up
  pinMode( SEC_LED, OUTPUT ); // Colon Blinking
  digitalWrite( SEC_LED, LOW ); // Enable internal pull down
  pinMode( NTP_LED, OUTPUT ); // NTP Status display
  digitalWrite( NTP_LED, LOW ); // Enable internal pull down
  
  // Switch Initial Setup
  pinMode(SW_CH1,INPUT);
  digitalWrite(SW_CH1,HIGH); // Activate Pull-Up
  pinMode(SW_CH2,INPUT);
  digitalWrite(SW_CH2,HIGH); // Activate Pull-Up
  pinMode(SW_CH3,INPUT);
  digitalWrite(SW_CH3,HIGH); // Activate Pull-Up
  pinMode(SW_CH4,INPUT);
  digitalWrite(SW_CH4,HIGH); // Activate Pull-Up
  pinMode(SW_CH5,INPUT);
  digitalWrite(SW_CH5,HIGH); // Activate Pull-Up
  
  // RGB Initial Setup
  pinMode(LED_ENA,INPUT);
  digitalWrite(LED_ENA,HIGH); // Activate Pull-Up
  pinMode(LED_PIN_RED, OUTPUT); 
  pinMode(LED_PIN_GREEN, OUTPUT);
  pinMode(LED_PIN_BLUE, OUTPUT);
  
  analogReference(EXTERNAL); //use external 5v reference
  //analogReference(INTERNAL2V56); //use internal 2.54v reference
  
  randomSeed(analogRead(7)); // RandamSeed
  
  // read eeprom values unless there is a jumper between RESET_PIN and GND:
  //if (eeprom_read_byte((uint8_t *)0x0) == 60) { // ok magic number matches accept values
  if (digitalRead(RESET_PIN) == HIGH && eeprom_read_byte((uint8_t *)0x0) == 60) {
    eeprom_read_block((uint8_t *)ip_address, (void *)1, sizeof(ip_address));
    eeprom_read_block((uint8_t *)netmask, (void *)6, sizeof(netmask));
    eeprom_read_block((uint8_t *)gw_addrsss, (void *)11, sizeof(gw_addrsss));
    eeprom_read_block((uint8_t *)dns_address, (void *)16, sizeof(dns_address));
    eeprom_read_block((uint8_t *)ntp_polling_address, (void *)21, sizeof(ntp_polling_address));
    hours_offset_to_utc = (int8_t)eeprom_read_byte((uint8_t *)26);
    clock_disp = (int8_t)eeprom_read_byte((uint8_t *)27);
    ampm = (int8_t)eeprom_read_byte((uint8_t *)28);
    date_disp_int |= (byte)eeprom_read_byte((uint8_t *)29) << 8;
    date_disp_int |= (byte)eeprom_read_byte((uint8_t *)30);
    date_disp_sec = (int8_t)eeprom_read_byte((uint8_t *)31);
    colon_disp = (int8_t)eeprom_read_byte((uint8_t *)32);
    day_time_start = (int8_t)eeprom_read_byte((uint8_t *)33);
    day_time_end = (int8_t)eeprom_read_byte((uint8_t *)34);
    day_brightness = (int8_t)eeprom_read_byte((uint8_t *)35);
    dimmer = (int8_t)eeprom_read_byte((uint8_t *)36);
    night_mode = (int8_t)eeprom_read_byte((uint8_t *)37);
    night_brightness = (int8_t)eeprom_read_byte((uint8_t *)38);
    //network = (int8_t)eeprom_read_byte((uint8_t *)39);
    dhcp = (int8_t)eeprom_read_byte((uint8_t *)40);
    ntp = (int8_t)eeprom_read_byte((uint8_t *)41);
    ntp_interval = (int8_t)eeprom_read_byte((uint8_t *)42);
    //temp_disp = (int8_t)eeprom_read_byte((uint8_t *)41);
    cefa = (int8_t)eeprom_read_byte((uint8_t *)41);
    
    LED_mode = (int8_t)eeprom_read_byte((uint8_t *)50);
    RGB_change_speed = (int8_t)eeprom_read_byte((uint8_t *)51);
    LED_red = (int8_t)eeprom_read_byte((uint8_t *)52);
    LED_green = (int8_t)eeprom_read_byte((uint8_t *)53);
    LED_blue = (int8_t)eeprom_read_byte((uint8_t *)54);
    //LED_light_time_start = (int8_t)eeprom_read_byte((uint8_t *)55);
    //LED_light_time_end = (int8_t)eeprom_read_byte((uint8_t *)56);
    
    DEBUG_PRINTLN("Reading Config"); // debug print

  } else {
    DEBUG_PRINTLN("Resetting Config"); // debug print
  }
  // Initial HV_hz
  pinMode(HV_PWM_PIN, OUTPUT);
  digitalWrite(HV_PWM_PIN, LOW);
  TCCR2A = _BV(COM2A0) | _BV(WGM21) | _BV(WGM20);
  TCCR2B = _BV(WGM22) | _BV(CS20);
  OCR2A = 128; // CPU_Mhz/128
  OCR2B = OCR2A / 2; // 50%-Duty
  
  // HV_PWM_hz Setup
  /*boolean HV_Flag;
  int HV_Count = 0;
  float HV_Volt = 0;

  while ( !(HV_Count > 16 && int(HV_Volt) < HV_SET_VOLT) ) {
  //while (int(HV_Volt) != HV_SET_VOLT || ( HV_Count < 16 && int(HV_Volt) > HV_SET_VOLT ) ) {
    if (int(HV_Volt) < HV_SET_VOLT ) {
      if ( OCR2A != 255 ) {
        OCR2A = OCR2A + 1;
        if ( !HV_Flag ) ++HV_Count;
        HV_Flag = true;
      } else {
        continue;
      }
    } else {
      if ( OCR2A != 0 ) {
        OCR2A = OCR2A - 1;
        if ( HV_Flag ) ++HV_Count;
        HV_Flag = false;
      } else {
        continue;
      }
    }
    OCR2B = OCR2A / 2; // 50%-Duty
    
    delay(250); // ResponseTime

    HV_Volt = float(analogRead( HV_SEN_PIN )) * float(HV_ADJ) * 200 / 1024; // VoltageCheck

    // Debug Print
    DEBUG_PRINT( "OCR2A:" );
    DEBUG_PRINT( OCR2A );
    DEBUG_PRINT( " OCR2B:" );
    DEBUG_PRINT( OCR2B );
    DEBUG_PRINT( " HV_Count:" );
    DEBUG_PRINT( HV_Count );
    DEBUG_PRINT( " PWM_hz:" );
    DEBUG_PRINT( F_CPU/OCR2A);
    DEBUG_PRINT( " HV_Val:" );
    DEBUG_PRINT( analogRead( HV_SEN_PIN ) );
    DEBUG_PRINT( " HV_Volt:" );
    DEBUG_PRINTLN( HV_Volt );
  }*/
  
  // Module Staartup
  DEBUG_PRINTLN("InutModule Started..."); // debug print
  
  // Setup MAX72XX
  // power-saving mode on startup, we have to do a wakeup call
  lc.shutdown( 0, false );  // LED Waik up
  lc.setIntensity( 0, day_brightness ); // Set the brightness
  lc.clearDisplay( 0 ); // and clear the display
  
  lc.setChar( 0, 0, ' ', true ); // InitialIndicater
  
  // initialize enc28j60
  es.ES_enc28j60SpiInit();
  
  uint8_t haveGWip = 1;
  
  es.ES_enc28j60Init(mac_address);
  DEBUG_PRINTLN("Init ENC28J60 done"); // debug print
  DEBUG_PRINT( "ENC28J60 version " ); // debug print
  DEBUG_PRINTLNH( es.ES_enc28j60Revision() , HEX ); // debug print
  DEBUG_PRINT( "Mac Address = " );
  if( es.ES_enc28j60Revision() <= 0 ) {
    DEBUG_PRINTLN( "Failed to access ENC28J60"); // debug print
    while(1);    // Just loop here
  } else {
    for( int i=0; i<6; i++ ) {
    DEBUG_PRINTH( mac_address[i], HEX ); // debug print
    DEBUG_PRINT( i < 5 ? ":" : "" ); // debug print
    }
    DEBUG_PRINTLN(""); // debug print
  }
  
  // Start dump process by passing Serial object and flags
  dump.begin( &Serial,true, true, false, true );
  
  // just use variable haveGWip as a loop counter here to save memory
  while ( haveGWip++ < 15 ) {
    //Serial.print("."); // debug print
    if (es.ES_enc28j60linkup()) {
      haveGWip = 99;
    }
    delay( 400 );
  }
  haveGWip = 0;
  //Serial.println(""); // debug print
  
  lc.setChar( 0, 1, ' ', true ); // InitialIndicater
  
  // Timer set 1 second
  attachInterrupt(1, CLK_SEC, RISING);
  //attachInterrupt(1, CLK, FALLING);
  
  // LinkCheck & Start
  if(es.ES_enc28j60linkup()) {
  //if(es.ES_enc28j60linkup() && network) 
    lc.setChar( 0, 2, ' ', true ); // InitialIndicater
    delay( 1000 );
    
    if ( !dhcp ) {
      //init the ethernet/ip layer:
      es.ES_init_ip_arp_udp_tcp(mac_address, ip_address, MYWWWPORT);
      
      // init the web client:
      es.ES_client_set_gwip(gw_addrsss);  // e.g internal IP of dsl router
      es.ES_dnslkup_set_dnsip(dns_address); // generally same IP as router
      
      // IP-address & Default-GW Display
      DEBUG_PRINT("IP Address:"); // debug print
      ip2str( strbuf, ip_address );
      DEBUG_PRINTLN( strbuf ); // debug print
      DEBUG_PRINT("Netmask:"); // debug print
      ip2str( strbuf, netmask );
      DEBUG_PRINTLN( strbuf ); // debug print
      lc.setChar( 0, 3, ' ', true ); // InitialIndicater
      delay( 1500 );
      DEBUG_PRINT("DefaultGateway:"); // debug print
      ip2str( strbuf, gw_addrsss );
      DEBUG_PRINTLN( strbuf ); // debug print
      lc.setChar( 0, 4, ' ', true ); // InitialIndicater
      delay( 1500 );
    } else {
      lc.setChar( 0, 5, ' ', true ); // InitialIndicater
      DEBUG_PRINTLN("DHCP Enable"); // debug print
      delay( 1500 );
    }
    
  } else {
    DEBUG_PRINTLN("Ethernet Down"); // debug print
    delay( 1500 );
  }
  if (!clock_disp) {
    lc.shutdown( 0, true );
  }
  lcd_needs_update = 60;
  
  // Setup DS18x20
  if ( ds.search(addr)) {
    DEBUG_PRINTLN("Init DS18x20 done");
    if (ds.crc8(addr, 7) == addr[7]) {

      // the first ROM byte indicates which chip
      switch (addr[0]) {
        case 0x10:
          DEBUG_PRINTLN("Chip = DS18S20");  // or old DS1820
          type_s = 1;
          break;
        case 0x28:
          DEBUG_PRINTLN("Chip = DS18B20");
          type_s = 0;
          break;
        case 0x22:
          DEBUG_PRINTLN("Chip = DS1822");
          type_s = 0;
          break;
        default:
          DEBUG_PRINTLN("Device is not a DS18x20 family device.");
          return;
      }
      DEBUG_PRINT("ROM = ");
      for( uint8_t i = 0; i < 8; i++) {
        DEBUG_PRINTH(addr[i], HEX);
      }
      DEBUG_PRINTLN("");
      
    } else {
      DEBUG_PRINTLN("CRC is not valid!");
      return;
    }
    
  } else {
    DEBUG_PRINTLN("No more addresses.");
    delay( 1500 );
  }
  
  DEBUG_PRINTLN("Init Setup End"); // debug print
  DEBUG_PRINTLN(""); // debug print
}
